package function.builtin;

import java.math.BigInteger;

import function.*;
import sexpression.*;

public class DIVIDE extends LispFunction {

    private ArgumentValidator argumentValidator;

    public DIVIDE() {
        this.argumentValidator = new ArgumentValidator("/");
        this.argumentValidator.setMinimumNumberOfArguments(1);
        this.argumentValidator.setEveryArgumentExpectedType(LispNumber.class);
    }

    public SExpression call(Cons argumentList) {
        argumentValidator.validate(argumentList);

        return callTailRecursive(argumentList);
    }

    private SExpression callTailRecursive(Cons argumentList) {
        Cons remainingArguments = (Cons) argumentList.getCdr();
        SExpression firstArgument = argumentList.getCar();
        LispNumber number1 = (LispNumber) firstArgument;

        if (remainingArguments.nullp())
            return getReciprocal(number1);

        SExpression secondArgument = remainingArguments.getCar();
        LispNumber number2 = (LispNumber) secondArgument;
        LispNumber quotient = divide(number1, number2);
        SExpression remainingNumbers = remainingArguments.getCdr();

        if (remainingNumbers.nullp())
            return quotient;

        return callTailRecursive(new Cons(quotient, remainingNumbers));
    }

    private LispNumber getReciprocal(LispNumber number1) {
        return new LispNumber(BigInteger.ONE.divide(number1.getValue()));
    }

    private LispNumber divide(LispNumber number1, LispNumber number2) {
        return new LispNumber(number1.getValue().divide(number2.getValue()));
    }

}
